/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplicationClientLoader.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.client.xml;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import org.ow2.easybeans.deployment.xml.parsing.EJB3EntityResolver;
import org.ow2.easybeans.deployment.xml.parsing.ParsingException;
import org.ow2.easybeans.deployment.xml.struct.common.EJBRef;
import org.ow2.easybeans.deployment.xml.struct.common.InjectionTarget;
import org.ow2.easybeans.util.url.URLUtilsException;
import org.ow2.easybeans.util.xml.DocumentParser;
import org.ow2.easybeans.util.xml.DocumentParserException;
import org.ow2.easybeans.util.xml.XMLUtils;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import static org.ow2.easybeans.util.url.URLUtils.fileToURL2;

/**
 * Class used to fill ApplicationClient implementation class by loading an XML.
 * @author Florent Benoit
 */
public final class ApplicationClientLoader {

    /**
     * Persistence namespace.
     */
    private static final String JAVAEE_NS = "http://java.sun.com/xml/ns/javaee";

    /**
     * &lt;ejb-ref&gt; element.
     */
    private static final String EJB_REF = "ejb-ref";

    /**
     * Directory where application-client.xml file should be.
     */
    private static final String DIRECTORY_APPLICATION_CLIENT_XML_FILE = "META-INF";

    /**
     * Name of the persistence.xml file.
     */
    private static final String APPLICATION_CLIENT_XML_FILE = "application-client.xml";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(ApplicationClientLoader.class);

    /**
     * Validating with schema ?
     */
    private static boolean validating = true;

    /**
     * Utility class, no public constructor.
     */
    private ApplicationClientLoader() {

    }

    /**
     * Load the application-client.xml file.
     * @param url the URL of the the Reader of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    private static ApplicationClient loadXML(final URL url) throws ParsingException {

        logger.debug("Analyzing url {0}", url);

        // Object that will be returned
        ApplicationClient applicationClient = new ApplicationClient();

        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(url, validating, new EJB3EntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <application-client>
        Element applicationClientRootElement = document.getDocumentElement();

        NodeList ejbRefList = applicationClientRootElement.getElementsByTagNameNS(JAVAEE_NS, EJB_REF);

        // Loop on this list
        for (int i = 0; i < ejbRefList.getLength(); i++) {
            Element ejbRefElement = (Element) ejbRefList.item(i);
            EJBRef ejbRef = new EJBRef();

            // ejb-ref-name
            String ejbRefName = XMLUtils.getStringValueElement(JAVAEE_NS, ejbRefElement, "ejb-ref-name");
            ejbRef.setEjbRefName(ejbRefName);

            // ejb-ref-type
            String ejbRefType = XMLUtils.getStringValueElement(JAVAEE_NS, ejbRefElement, "ejb-ref-type");
            ejbRef.setEjbRefType(ejbRefType);

            // home
            String home = XMLUtils.getStringValueElement(JAVAEE_NS, ejbRefElement, "home");
            ejbRef.setHome(home);

            // remote
            String remote = XMLUtils.getStringValueElement(JAVAEE_NS, ejbRefElement, "remote");
            ejbRef.setRemote(remote);

            // ejb-link
            String ejbLink = XMLUtils.getStringValueElement(JAVAEE_NS, ejbRefElement, "ejb-link");
            ejbRef.setEjbLink(ejbLink);

            // injection target
            NodeList injectionTargetList = ejbRefElement.getElementsByTagNameNS(JAVAEE_NS, "injection-target");
            for (int j = 0; j < injectionTargetList.getLength(); j++) {
                Element injectionTargetElement = (Element) injectionTargetList.item(j);

                // Build Object and add it
                InjectionTarget injectionTarget = new InjectionTarget();
                ejbRef.addInjectionTarget(injectionTarget);

                // Get class name
                String className = XMLUtils.getStringValueElement(JAVAEE_NS, injectionTargetElement, "injection-target-class");
                injectionTarget.setClassname(className);

                // Get target
                String targetName = XMLUtils.getStringValueElement(JAVAEE_NS, injectionTargetElement, "injection-target-name");
                injectionTarget.setTargetName(targetName);

            }

            // add ejb-ref in application-client element
            applicationClient.addEJBRef(ejbRef);
        }
        return applicationClient;
    }

    /**
     * Detects and analyze the META-INF/application-client.xml file.
     * @param archive the file to analyze (or directory) in order to find a
     *        application-client.xml file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static ApplicationClient loadApplicationClient(final File archive) throws ParsingException {

        URL applicationClientXmlURL = null;
        // if archive is a file, needs to look into the jar file
        if (archive.isFile()) {

            // Try to see if the archive contains the persistence.xml file.
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(archive);
            } catch (IOException e) {
                throw new ParsingException("File '" + jarFile + "' is not a valid jar file.", e);
            }

            // Gets the entry.
            String applicationClientEntry = DIRECTORY_APPLICATION_CLIENT_XML_FILE + '/' + APPLICATION_CLIENT_XML_FILE;
            ZipEntry zipEntry = jarFile.getEntry(applicationClientEntry);
            if (zipEntry != null) {
                String jarUrl = "jar:file:" + archive.getPath() + "!/" + applicationClientEntry;
                try {
                    applicationClientXmlURL = new URL(jarUrl);
                } catch (MalformedURLException e) {
                    throw new ParsingException("Error while trying to build an URL with '" + jarUrl + "'", e);
                }
            }
            try {
                jarFile.close();
            } catch (IOException e) {
                logger.error("Problem while closing jar file '" + jarFile + "'.");
            }

        } else {
            // directory mode
            File applicationClientXmlFile = new File(archive, DIRECTORY_APPLICATION_CLIENT_XML_FILE + File.separator
                    + APPLICATION_CLIENT_XML_FILE);
            if (applicationClientXmlFile.exists()) {
                try {
                    applicationClientXmlURL = fileToURL2(applicationClientXmlFile);
                } catch (URLUtilsException e) {
                    throw new ParsingException("Cannot get URL from file '" + applicationClientXmlFile + "'.");
                }
            }
        }

        // Now, do the parsing and fill the structure.
        ApplicationClient applicationClient = null;
        if (applicationClientXmlURL != null) {
            try {
                applicationClient = loadXML(applicationClientXmlURL);
            } catch (ParsingException e) {
                throw new ParsingException("Cannot parse the URL '" + applicationClientXmlURL + "'.", e);
            }
        }
        return applicationClient;
    }

}
