/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:EjbJarClassMetadata.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.metadata.ejbjar;

import static javax.ejb.TransactionAttributeType.REQUIRED;
import static javax.ejb.TransactionManagementType.CONTAINER;
import static org.ow2.easybeans.deployment.metadata.CommonMetadata.concatStringBuilder;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.ow2.easybeans.deployment.annotations.ClassType;
import org.ow2.easybeans.deployment.annotations.InterceptorType;
import org.ow2.easybeans.deployment.annotations.JClassInterceptor;
import org.ow2.easybeans.deployment.annotations.exceptions.InterceptorsValidationException;
import org.ow2.easybeans.deployment.annotations.impl.JCommonBean;
import org.ow2.easybeans.deployment.annotations.impl.JInterceptors;
import org.ow2.easybeans.deployment.annotations.impl.JLocal;
import org.ow2.easybeans.deployment.annotations.impl.JMessageDriven;
import org.ow2.easybeans.deployment.annotations.impl.JRemote;
import org.ow2.easybeans.deployment.annotations.impl.JStateful;
import org.ow2.easybeans.deployment.annotations.impl.JStateless;
import org.ow2.easybeans.deployment.metadata.CommonClassMetadata;
import org.ow2.easybeans.deployment.metadata.interfaces.IAnnotationSecurityPermitAll;
import org.ow2.easybeans.deployment.metadata.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.easybeans.deployment.metadata.interfaces.IEJBInterceptors;
import org.ow2.easybeans.deployment.metadata.interfaces.IInterceptorExcludeDefaultInterceptors;
import org.ow2.easybeans.deployment.metadata.interfaces.ITransactionAttribute;
import org.ow2.util.pool.api.IPoolConfiguration;
import org.ow2.util.pool.api.IPoolMetadata;
import org.ow2.util.scan.api.metadata.structures.JClass;


/**
 * This class represents the annotation metadata of a Bean.<br>
 * From this class, we can access to all methods of a bean with its associated information.
 * @author Florent Benoit
 */
public class EjbJarClassMetadata extends CommonClassMetadata<EjbJarClassMetadata, EjbJarMethodMetadata, EjbJarFieldMetadata>
        implements ITransactionAttribute, IEJBInterceptors, IAnnotationSecurityRolesAllowed, IAnnotationSecurityPermitAll,
        IInterceptorExcludeDefaultInterceptors, IPoolMetadata {

    /**
     * Logger.
     */
    // private static Log logger = LogFactory.getLog(EjbJarClassMetadata.class);

    /**
     * Parent meta data.
     */
    private EjbJarArchiveMetadata ejbJarArchiveMetadata = null;

    /**
     * List of local interfaces.
     */
    private JLocal jLocal = null;

    /**
     * List of remote interfaces.
     */
    private JRemote jRemote = null;

    /**
     * CommonBean description.
     */
    private JCommonBean jCommonBean = null;

    /**
     * Message Driven attribute.
     */
    private JMessageDriven jMessageDriven = null;

    /**
     * Stateless attribute.
     */
    private JStateless jStateless = null;

    /**
     * Stateful attribute.
     */
    private JStateful jStateful = null;

    /**
     * Ordered interceptors. If this flag is true it means that
     * interceptor-binding has been used in DD with an interceptor-order
     * element.<br/>
     * Also, this flag is used to check that the ordering is done only once.
     */
    private boolean orderedInterceptors = false;

    /**
     * Local Home.
     */
    private String localHome = null;

    /**
     * Remote Home.
     */
    private String remoteHome = null;

    /**
     * List of annotation interceptors.
     */
    private JInterceptors annotationInterceptors = null;

    /**
     * EasyBeans global interceptors.<br>
     * These interceptors correspond to a list of interceptors
     * that need to be present first on all methods.
     */
    private List<JClassInterceptor> globalEasyBeansInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in its bean class.
     * It is the interceptors defined in interceptor classes, not the bean class itself.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<JClassInterceptor>> externalUserInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in the bean class.
     * It is not defined in separated classes.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<JClassInterceptor>> internalUserInterceptors = null;


    /**
     * Transaction management type (default = container).
     */
    private TransactionManagementType transactionManagementType = CONTAINER;

    /**
     * Transaction attribute type (default = required).
     */
    private TransactionAttributeType transactionAttributeType = REQUIRED;

    /**
     * Application exception annotation.
     */
    private ApplicationException applicationException = null;


    /**
     * The type of the class.
     * @see ClassType
     */
    private ClassType classType = null;

    /**
     * List of &#64;{@link javax.interceptor.AroundInvoke} methods on this
     * class (should be only one per class, validating occurs after).
     */
    private List<EjbJarMethodMetadata> aroundInvokeMethodsMetadata = null;


    /**
     * Methods used for &#64;{@link javax.annotation.PostConstruct} on this
     * class (only one per class but may be defined in super classes).
     */
    private LinkedList<EjbJarMethodMetadata> postConstructMethodsMetadata = null;

    /**
     * Methods used for &#64;{@link javax.annotation.PreDestroy} on this class
     * (only one per class but may be defined in super classes).
     */
    private LinkedList<EjbJarMethodMetadata> preDestroyMethodsMetadata = null;

    /**
     * Methods used for &#64;{@link javax.ejb.PostActivate} on this class (only
     * one per class but may be defined in super classes).
     */
    private LinkedList<EjbJarMethodMetadata> postActivateMethodsMetadata = null;

    /**
     * Methods used for &#64;{@link javax.ejb.PrePassivate} on this class (only
     * one per class but may be defined in super classes).
     */
    private LinkedList<EjbJarMethodMetadata> prePassivateMethodsMetadata = null;

    /**
     * Is that the class represented by this metadata has already been modified ?
     */
    private boolean modified = false;

    /**
     * List of roles that are declared on this class.
     */
    private List<String> declareRoles = null;

    /**
     * List of roles that are allowed on this class/bean.
     */
    private List<String> rolesAllowed = null;

    /**
     * This class/bean has the permitAll annotation.
     */
    private boolean permitAll = false;

    /**
     * The run-as security role (if any).
     */
    private String runAs = null;

    /**
     * &#64;{@link javax.interceptor.ExcludeDefaultInterceptors} method ?
     */
    private boolean excludeDefaultInterceptors = false;

    /**
     * Name of the remote interface (used in EJB 2.1 DD).
     */
    private String remoteInterface = null;

    /**
     * Pool's configuration.
     */
    private IPoolConfiguration poolConfiguration;

    /**
     * Constructor.
     * @param jClass jClass.
     * @param ejbJarArchiveMetadata parent metadata object.
     */
    public EjbJarClassMetadata(final JClass jClass, final EjbJarArchiveMetadata ejbJarArchiveMetadata) {
        super(jClass);
        this.ejbJarArchiveMetadata = ejbJarArchiveMetadata;
        this.postConstructMethodsMetadata = new LinkedList<EjbJarMethodMetadata>();
        this.preDestroyMethodsMetadata = new LinkedList<EjbJarMethodMetadata>();
        this.postActivateMethodsMetadata = new LinkedList<EjbJarMethodMetadata>();
        this.prePassivateMethodsMetadata = new LinkedList<EjbJarMethodMetadata>();
    }

    /**
     * @return name of the bean (associated to this metadata).
     */
    public String getClassName() {
        return getJClass().getName();
    }

    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    public void setLocalInterfaces(final JLocal jLocal) {
        this.jLocal = jLocal;
    }

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    public void setRemoteInterfaces(final JRemote jRemote) {
        this.jRemote = jRemote;
    }


    /**
     * @return the local interfaces of this class.
     */
    public JLocal getLocalInterfaces() {
       return this.jLocal;
    }

    /**
     * @return the remote interfaces of this class.
     */
    public JRemote getRemoteInterfaces() {
        return this.jRemote;
    }

      /**
     * @return true if the class is a stateless class
     */
    public boolean isStateless() {
        return (this.classType != null && this.classType == ClassType.STATELESS);
    }

    /**
     * @return true if the class is a stateful class
     */
    public boolean isStateful() {
        return (this.classType != null && this.classType == ClassType.STATEFUL);
    }

    /**
     * @return true if the class is a session bean class
     */
    public boolean isSession() {
        return (this.classType != null && (this.classType == ClassType.STATELESS || this.classType == ClassType.STATEFUL));
    }

    /**
     * @return true if the class is an MDB class
     */
    public boolean isMdb() {
        return (this.classType != null && this.classType == ClassType.MDB);
    }

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.ow2.easybeans.deployment.annotations.ClassType
     */
    public void setClassType(final ClassType cType) {
        this.classType = cType;
    }

    /**
     * @return Message driven attribute.
     */
    public JMessageDriven getJMessageDriven() {
        return this.jMessageDriven;
    }

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    public void setJMessageDriven(final JMessageDriven messageDriven) {
        this.jMessageDriven = messageDriven;
    }


    /**
     * @return string representation.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Add super class toString()
        sb.append(super.toString());

        // Class name
        concatStringBuilder("className", getClassName(), sb);

        // superclass name
        concatStringBuilder("superName", getJClass().getSuperName(), sb);

        // interfaces
        concatStringBuilder("interfaces", getJClass().getInterfaces(), sb);

        // classType
        concatStringBuilder("classType", this.classType, sb);

        // jLocal
        concatStringBuilder("jLocal", this.jLocal, sb);

        // aroundInvokeMethodMetadatas
        concatStringBuilder("aroundInvokeMethodsMetadata", this.aroundInvokeMethodsMetadata, sb);

        // jRemote
        concatStringBuilder("jRemote", this.jRemote, sb);

        // jMessageDriven
        concatStringBuilder("jMessageDriven", this.jMessageDriven, sb);

        // remoteHome
        concatStringBuilder("remoteHome", this.remoteHome, sb);

        // localHome
        concatStringBuilder("localHome", this.localHome, sb);

        // transactionManagementType
        concatStringBuilder("transactionManagementType", this.transactionManagementType, sb);

        // transactionAttributeType
        concatStringBuilder("transactionAttributeType", this.transactionAttributeType, sb);

        // annotation Interceptors
        concatStringBuilder("annotationInterceptors", this.annotationInterceptors, sb);

        // jEjbEJBs
        concatStringBuilder("jAnnotationEJBs", getJEjbEJBs(), sb);

        // jAnnotationResources
        concatStringBuilder("jAnnotationResources", getJAnnotationResources(), sb);

        // javaxPersistencePersistenceContexts
        concatStringBuilder("javaxPersistencePersistenceContexts", getJavaxPersistencePersistenceContexts(), sb);

        // javaxPersistencePersistenceUnits
        concatStringBuilder("javaxPersistencePersistenceUnits", getJavaxPersistencePersistenceUnits(), sb);


        // Methods
        for (EjbJarMethodMetadata methodAnnotationMetadata : getMethodMetadataCollection()) {
            concatStringBuilder("methods", methodAnnotationMetadata, sb);
        }

        // modified
        concatStringBuilder("modified", Boolean.valueOf(this.modified), sb);

        // excludeDefaultInterceptors
        concatStringBuilder("excludeDefaultInterceptors", Boolean.valueOf(this.excludeDefaultInterceptors), sb);

        // declareRoles
        concatStringBuilder("declareRoles", this.declareRoles, sb);

        // rolesAllowed
        concatStringBuilder("rolesAllowed", this.rolesAllowed, sb);

        // permitAll
        concatStringBuilder("permitAll", Boolean.valueOf(this.permitAll), sb);

        // runAs
        concatStringBuilder("runAs", this.runAs, sb);

        sb.append("]");
        return sb.toString();
    }

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    public String getRemoteHome() {
        return this.remoteHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    public void setRemoteHome(final String remoteHome) {
        this.remoteHome = remoteHome;
    }

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    public String getLocalHome() {
        return this.localHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    public void setLocalHome(final String localHome) {
        this.localHome = localHome;
    }

    /**
     * @return transaction management type from @see TransactionManagementType.
     */
    public TransactionManagementType getTransactionManagementType() {
        return this.transactionManagementType;
    }

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value.
     *      (BEAN, CONTAINER)
     */
    public void setTransactionManagementType(final TransactionManagementType transactionManagementType) {
        this.transactionManagementType = transactionManagementType;
    }

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return this.transactionAttributeType;
    }

    /**
     * Set Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        this.transactionAttributeType = transactionAttributeType;
    }

    /**
     * @return object representing list of &#64;{@link javax.interceptor.Interceptors}.
     */
    public JInterceptors getAnnotationInterceptors() {
        return this.annotationInterceptors;
    }

    /**
     * Sets the object representing the &#64;{@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    public void setAnnotationsInterceptors(final JInterceptors annotationInterceptors) {
        this.annotationInterceptors = annotationInterceptors;
    }

    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    public ApplicationException getApplicationException() {
        return this.applicationException;
    }

    /**
     * Sets the object representing the &#64;{@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    public void setApplicationException(final ApplicationException applicationException) {
        this.applicationException = applicationException;
    }

    /**
     * @return true if the classs is a Bean
     */
    public boolean isBean() {
        return isStateless() || isStateful() || isMdb();
    }

    /**
     * @return array of interfaces name.
     */
    public String[] getInterfaces() {
        return getJClass().getInterfaces();
    }

    /**
     * @return the super class name.
     */
    public String getSuperName() {
        return getJClass().getSuperName();
    }

    /**
     * @return parent metadata object.
     */
    public EjbJarArchiveMetadata getEjbJarArchiveMetadata() {
        return this.ejbJarArchiveMetadata;
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getExternalUserEasyBeansInterceptors() {
        return this.externalUserInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined outside the bean class (interceptor classes).
     * @param externalUserInterceptors list of interceptors that enhancer will use.
     */
    public void setExternalUserInterceptors(final Map<InterceptorType, List<JClassInterceptor>> externalUserInterceptors) {
        this.externalUserInterceptors = externalUserInterceptors;
    }


    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (bean classes)
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getInternalUserEasyBeansInterceptors() {
        return this.internalUserInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param internalUserInterceptors list of interceptors that enhancer will use.
     */
    public void setInternalUserInterceptors(final Map<InterceptorType, List<JClassInterceptor>> internalUserInterceptors) {
        this.internalUserInterceptors = internalUserInterceptors;
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    public List<JClassInterceptor> getGlobalEasyBeansInterceptors() {
        return this.globalEasyBeansInterceptors;
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    public void setGlobalEasyBeansInterceptors(final List<JClassInterceptor> globalEasyBeansInterceptors) {
        this.globalEasyBeansInterceptors = globalEasyBeansInterceptors;
    }

    /**
     * @return the method metadata with annotation &#64;{@link javax.interceptor.AroundInvoke}.
     */
    public boolean isAroundInvokeMethodMetadata() {
        return (this.aroundInvokeMethodsMetadata != null);
    }


    /**
     * @return the list of methods metadata with annotation &#64;{@link javax.interceptor.AroundInvoke}.
     */
    public List<EjbJarMethodMetadata> getAroundInvokeMethodMetadatas() {
        return this.aroundInvokeMethodsMetadata;
    }

    /**
     * Add a &#64;{@link javax.interceptor.AroundInvoke} method of this class.
     * @param aroundInvokeMethodMetadata the method.
     */
    public void addAroundInvokeMethodMetadata(final EjbJarMethodMetadata aroundInvokeMethodMetadata) {
        if (this.aroundInvokeMethodsMetadata == null) {
            this.aroundInvokeMethodsMetadata = new ArrayList<EjbJarMethodMetadata>();
        }
        this.aroundInvokeMethodsMetadata.add(aroundInvokeMethodMetadata);
    }

    /**
     * @return the methods metadata with annotation &#64;{@link javax.annotation.PostConstruct}.
     */
    public LinkedList<EjbJarMethodMetadata> getPostConstructMethodsMetadata() {
        return this.postConstructMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.annotation.PostConstruct} method of this class.
     * @param postConstructMethodMetadata the method.
     */
    public void addPostConstructMethodMetadata(final EjbJarMethodMetadata postConstructMethodMetadata) {
      checkLifeCycleDuplicate(postConstructMethodMetadata, InterceptorType.POST_CONSTRUCT, getPostConstructMethodsMetadata());
      this.postConstructMethodsMetadata.addFirst(postConstructMethodMetadata);
    }

    /**
     * Checks that only method at one level of a class is present.
     * @param postConstructMethodMetadata method to check
     * @param itcType the type of interceptor (used for the error)
     * @param existingList current list of methods
     */
    private void checkLifeCycleDuplicate(final EjbJarMethodMetadata postConstructMethodMetadata,
            final InterceptorType itcType, final List<EjbJarMethodMetadata> existingList) {
        // First case : not inherited
        EjbJarClassMetadata wantToAddClassMetadata = postConstructMethodMetadata.getClassMetadata();
        if (postConstructMethodMetadata.isInherited()) {
            wantToAddClassMetadata = postConstructMethodMetadata.getOriginalClassMetadata();
        }
        for (EjbJarMethodMetadata method : existingList) {
            EjbJarClassMetadata compareMetaData;
            if (method.isInherited()) {
                compareMetaData = method.getOriginalClassMetadata();
            } else {
                compareMetaData = method.getClassMetadata();
            }
            if (compareMetaData.equals(wantToAddClassMetadata)) {
                throw new InterceptorsValidationException("Class " + getClassName()
                        + " has already a " + itcType + " method which is "
                        + method.getMethodName() + ", cannot set new method "
                        + postConstructMethodMetadata.getMethodName());
            }
        }
    }


    /**
     * @return the methods metadata with annotation &#64;{@link javax.annotation.PreDestroy}.
     */
    public LinkedList<EjbJarMethodMetadata> getPreDestroyMethodsMetadata() {
        return this.preDestroyMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.annotation.PreDestroy} method of this class.
     * @param preDestroyMethodMetadata the method.
     */
    public void addPreDestroyMethodMetadata(final EjbJarMethodMetadata preDestroyMethodMetadata) {
        checkLifeCycleDuplicate(preDestroyMethodMetadata, InterceptorType.PRE_DESTROY, getPreDestroyMethodsMetadata());
        this.preDestroyMethodsMetadata.addFirst(preDestroyMethodMetadata);
    }


    /**
     * @return the methods metadata with annotation &#64;{@link javax.ejb.PostActivate}.
     */
    public LinkedList<EjbJarMethodMetadata> getPostActivateMethodsMetadata() {
        return this.postActivateMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.ejb.PostActivate} method of this class.
     * @param postActivateMethodMetadata the method.
     */
    public void addPostActivateMethodMetadata(final EjbJarMethodMetadata postActivateMethodMetadata) {
        checkLifeCycleDuplicate(postActivateMethodMetadata, InterceptorType.POST_ACTIVATE, getPostActivateMethodsMetadata());
        this.postActivateMethodsMetadata.addFirst(postActivateMethodMetadata);
    }


    /**
     * @return the method metadata with annotation &#64;{@link javax.ejb.PrePassivate}.
     */
    public LinkedList<EjbJarMethodMetadata> getPrePassivateMethodsMetadata() {
        return this.prePassivateMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.ejb.PrePassivate} method of this class.
     * @param prePassivateMethodMetadata the method.
     */
    public void addPrePassivateMethodMetadata(final EjbJarMethodMetadata prePassivateMethodMetadata) {
        checkLifeCycleDuplicate(prePassivateMethodMetadata, InterceptorType.PRE_PASSIVATE, getPrePassivateMethodsMetadata());
        this.prePassivateMethodsMetadata.addFirst(prePassivateMethodMetadata);
    }


    /**
     * Is that this class is an interceptor class ?
     * @return true if it the case, else false.
     */
    public boolean isInterceptor() {
        return (this.aroundInvokeMethodsMetadata != null && this.aroundInvokeMethodsMetadata.size() > 0)
        || (this.postConstructMethodsMetadata != null && this.postConstructMethodsMetadata.size() > 0)
        || (this.preDestroyMethodsMetadata != null && this.preDestroyMethodsMetadata.size() > 0)
        || (this.prePassivateMethodsMetadata != null && this.prePassivateMethodsMetadata.size() > 0)
        || (this.postActivateMethodsMetadata != null && this.postActivateMethodsMetadata.size() > 0);
    }


    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    public JCommonBean getJCommonBean() {
        return this.jCommonBean;
    }

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    public void setJCommonBean(final JCommonBean commonBean) {
        // ensure that there is a bean name
        String ejbName = commonBean.getName();
        if (ejbName == null || "".equals(ejbName)) {
            // if not set, it has to be the unqualified name of the bean class. (simplified spec : 10.1.1)
            String className = getClassName();
            commonBean.setName(className.substring(className.lastIndexOf("/") + 1));
        }
        this.jCommonBean = commonBean;
    }

    /**
     * @return the attributes for a Stateful
     */
    public JStateful getJStateful() {
        return this.jStateful;
    }

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    public void setJStateful(final JStateful jStateful) {
        this.jStateful = jStateful;
    }

    /**
     * @return the attributes for a Stateless
     */
    public JStateless getJStateless() {
        return this.jStateless;
    }

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    public void setJStateless(final JStateless jStateless) {
        this.jStateless = jStateless;
    }

    /**
     * @return true if the class has been modified.
     */
    public boolean wasModified() {
        return this.modified;
    }

    /**
     * Defines that this class has been modified.
     */
    public void setModified() {
        this.modified = true;
    }

    /**
     * Sets the list of roles declared on this class.
     * @param declareRoles the list of roles.
     */
    public void setDeclareRoles(final List<String> declareRoles) {
        this.declareRoles = declareRoles;
    }

    /**
     * @return the list of roles declared on this class.
     */
    public List<String> getDeclareRoles() {
        return this.declareRoles;
    }

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        this.rolesAllowed = rolesAllowed;
    }

    /**
     * @return the list of roles allowed on this class/method.
     */
    public List<String> getRolesAllowed() {
        return this.rolesAllowed;
    }

    /**
     * This class has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    public void setPermitAll(final boolean permitAll) {
        this.permitAll = permitAll;
    }

    /**
     * @return true if PermitAll annotation.
     */
    public boolean hasPermitAll() {
        return this.permitAll;
    }

    /**
     * Set the value of the run-as property.
     * @param runAs the run-as property.
     */
    public void setRunAs(final String runAs) {
        this.runAs = runAs;
    }

    /**
     * @return the name of the security-role of the run-as element.
     */
    public String getRunAs() {
        return this.runAs;
    }


    /**
     * @return true if this class won't use default interceptors.
     */
    public boolean isExcludedDefaultInterceptors() {
        return this.excludeDefaultInterceptors;
    }

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which exclude default interceptors.
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        this.excludeDefaultInterceptors = excludeDefaultInterceptors;
    }

    /**
     * @return true if the interceptors have been ordered for this class.
     */
    public boolean isOrderedInterceptors() {
        return this.orderedInterceptors;
    }

    /**
     * Sets flag to true if interceptors are ordered.
     * @param orderedInterceptors boolean with true/false.
     */
    public void setOrderedInterceptors(final boolean orderedInterceptors) {
        this.orderedInterceptors = orderedInterceptors;
    }

    /**
     * Set the value of the remote Interface property.
     * @param remoteInterface the remote Interface property.
     */
    public void setRemoteInterface(final String remoteInterface) {
        this.remoteInterface = remoteInterface;
    }

    /**
     * @return the name of the remote interface.
     */
    public String getRemoteInterface() {
        return this.remoteInterface;
    }

    /**
     * construct a new JClass.
     * @param array interface list
     */
    public void setInterfaces(final String[] array) {
        JClass jClass = getJClass();
        setJClass(new JClass(jClass.getVersion(), jClass.getAccess(), jClass.getName(), jClass.getSignature(), jClass
                .getSuperName(), array));
    }


    /**
     * @return the pool configuration for this metadata.
     */
    public IPoolConfiguration getPoolConfiguration() {
        return this.poolConfiguration;
    }

    /**
     * Sets the pool configuration for this metadata.
     * @param poolConfiguration the given configuration.
     */
    public void setPoolConfiguration(final IPoolConfiguration poolConfiguration) {
        this.poolConfiguration = poolConfiguration;
    }

}
