/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WARTLDLoader.java 2071 2007-11-27 18:25:33Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.xml.parsing;

import java.net.URL;

import org.ow2.easybeans.deployment.xml.struct.Tag;
import org.ow2.easybeans.deployment.xml.struct.WAR;
import org.ow2.easybeans.util.xml.DocumentParser;
import org.ow2.easybeans.util.xml.DocumentParserException;
import org.ow2.easybeans.util.xml.XMLUtils;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class analyzes the TLD files and fill a given struct.
 * @author Florent Benoit
 */
public final class WARTLDLoader extends AbsWebLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WARTLDLoader.class);

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the tld file.
     * @param war the given war file that needs to be completed.
     */
    private WARTLDLoader(final URL url, final WAR war) {
        super(url, war);
    }

    /**
     * Load the tld file.
     * @param url the URL of the TLD file.
     * @param war the given war file that needs to be completed.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static WAR scanTLD(final URL url, final WAR war) throws ParsingException {
        logger.debug("Analyzing url {0}", url);
        WARTLDLoader warTldLoader = new WARTLDLoader(url, war);
        warTldLoader.parse();
        return warTldLoader.getWAR();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), false, new WAREntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <taglib>
        Element tagLibRootElement = document.getDocumentElement();

        // Analyze the taglib element
        analyzeListeners(tagLibRootElement);
        analyzeTags(tagLibRootElement);

    }

    /**
     * Find and analyze the tags in the given DOM element.
     * @param rootElement the element to analyze
     */
    protected void analyzeTags(final Element rootElement) {
        // Listeners
        NodeList tagList = rootElement.getElementsByTagName(Tag.NAME);
        for (int i = 0; i < tagList.getLength(); i++) {
            Element tagElement = (Element) tagList.item(i);

            // Build instance of a Tag struct object
            Tag tag = new Tag();
            getWAR().addTag(tag);

            // Analyze tag
            analyzeTag(tagElement, tag);
        }
    }


    /**
     * Analyze the given tag.
     * @param tagElement the dom element.
     * @param tag the structure representing object.
     */
    protected void analyzeTag(final Element tagElement, final Tag tag) {
        // Get class
        String tagClassName = XMLUtils.getStringValueElement(tagElement, "tag-class");
        tag.setTagClassName(tagClassName);
    }

}
